
#pragma once

#include "stdafx.h"

using namespace System;
using namespace System::Reflection;

#using <CSharp External DLL.dll>

// The following helper class allows the .Net assembly to be loaded from the same
// folder in which the mixed-mode DLL (this DLL) is located. It is assumed that this
// is how a typical user (of GoldSim) would organize its External element code. 
// Why is this necessary? .Net DLLs (.Net assemblies) are searched in a specific way.
// They may be registered in the GAC, or they may be located in the folder of the 
// executing application. However, for GoldSim External DLLs this is not what an end
// user would typically do. Instead, they would keep the DLLs with the model file, and
// pass all of those on to other users or their clients when they share their work.
// NOTE: The default release code optimization apparently removes this code. I had 
// to disable optimization for this FILE to make it work (/Od). Also, this file is 
// compiled for usage with the Common Language Runtime (/clr).
public ref class ManagedDLLHelper
{
public:
	static void PrepareAssembly()
	{
		// Register event handler to allow for custom rule when .Net assembly is loaded.
		AppDomain^ currentDomain = AppDomain::CurrentDomain;
		currentDomain->AssemblyResolve += gcnew ResolveEventHandler(&ManagedDLLHelper::AssemblyResolveEventHandler);
	}

private:

	static String^ GetDllPath()
	{
		// Determine the path of the mixed-mode DLL (this DLL).
		String^ myLocation = ManagedDLLHelper::typeid->Assembly->Location;
		return System::IO::Path::GetDirectoryName(myLocation);

	}

	static Assembly^ AssemblyResolveEventHandler(Object^ sender, ResolveEventArgs^ args)
	{
		// Manually load the assembly here by correcting the path to match the location of the mixed-mode DLL (this DLL).
		String^ CorrectedPath = GetDllPath() + "\\" + "CSharp External DLL.dll";
		return Assembly::LoadFrom(CorrectedPath);
	}
};


//
// Prototypes for calls to managed functions in .Net assembly
//
double ManagedSum(double op1, double op2)
{
	return CSharp_External_DLL::ClassForGoldSim::CSharpSum(op1, op2);
}

double ManagedMult(double op1, double op2)
{
	return CSharp_External_DLL::ClassForGoldSim::CSharpMult(op1, op2);
}



//
// Native C/C++ interface code to communicate with GoldSim.
//

// Define GoldSim function call methods
#define XF_INITIALIZE			0
#define XF_CALCULATION			1
#define XF_REP_VERSION			2
#define XF_REP_ARGUMENTS		3
#define XF_CLEANUP				99
#define XF_FATAL				1

// Export native function to add two numbers.
extern "C" __declspec(dllexport)
void MixedSum (int XFmethod, int *XFState, const double inarg[], double *outarg)
{
	*XFState = 0;

	// Load .Net assembly from same folder as this mixed-mode (native & managed) DLL.
	ManagedDLLHelper::PrepareAssembly(); 

	switch (XFmethod) 
	{
	case  XF_INITIALIZE:	// No initialization required
		break;				

	case  XF_REP_VERSION:
		outarg[0] = 1.0;	// Report version 1.0
		break;

	case  XF_REP_ARGUMENTS:
		outarg[0] = 2.0;   // 2 incoming numbers
		outarg[1] = 1.0;   // 1 result
		break;

	case  XF_CALCULATION:
		outarg[0] = ManagedSum(inarg[0], inarg[1]);
		break;

	case  XF_CLEANUP:		// No clean-up required
		break;
	}
} 


// Export native function to multiple two numbers.
extern "C" __declspec(dllexport)
void MixedMult (int XFmethod, int *XFState, const double inarg[], double *outarg)
{
	*XFState = 0;

	// Load .Net assembly from same folder as this mixed-mode (native & managed) DLL.
	ManagedDLLHelper::PrepareAssembly(); 

	switch (XFmethod) 
	{
	case  XF_INITIALIZE:	// No initialization required
		break;				

	case  XF_REP_VERSION:
		outarg[0] = 1.0;	// Report version 1.0
		break;

	case  XF_REP_ARGUMENTS:
		outarg[0] = 2.0;   // 2 incoming numbers
		outarg[1] = 1.0;   // 1 result
		break;

	case  XF_CALCULATION:
		outarg[0] = ManagedMult(inarg[0], inarg[1]);
		break;

	case  XF_CLEANUP:		// No clean-up required
		break;
	}
} 